@@ -2,67 +2,180 @@
 ##
 ## Internal function to remove duplicate and complement rules:
 ##
-delete_duplicates_complements <- function(rules, data, removecomplements = TRUE, 
-                                          removeduplicates = TRUE, 
-                                          return.dupl.compl = FALSE) {
-  
-  
+delete_duplicates_complements <- function(
+  rules, data, removecomplements = TRUE, removeduplicates = TRUE,
+  return.dupl.compl = FALSE, sparse = FALSE, keep_rulevars = FALSE) {
   ## Generate rule variables:
-  expr <- parse(text = paste0("cbind(", paste0(rules, collapse = ", "), ")"))
-  rulevars <- eval(expr, data)
-  colnames(rulevars) <- names(rules) <- paste0("rule", 1:length(rules))
+  rulevars <- if(sparse)
+    .get_rules_mat_sparse(data, rules) else
+      .get_rules_mat_dense(data, rules)
+  rules_to_process <- length(rules) > 0
+  if (rules_to_process) {
+    colnames(rulevars) <- names(rules) <- paste0("rule", 1:length(rules))
+  }
   
-  ## Remove duplicate rules:
-  if (removeduplicates) {
-    # Remove rules with identical support:
+  ## Remove duplicate rules
+  if (removeduplicates && rules_to_process) {
+    # Remove rules with identical support
     duplicates <- duplicated(rulevars, MARGIN = 2)
     duplicates.removed <- rules[duplicates]
     rulevars <- rulevars[, !duplicates, drop = FALSE]
     rules <- rules[!duplicates]
-  } else {
+    
+  } else 
     duplicates.removed <- NULL
-  }
   
-  
-  ## Remove complement rules:
-  if (removecomplements) {
-    # Get variance of each rule:
-    vars <- apply(rulevars, 2, var_bin)
-    vars_distinct <- sapply(
-      unique(vars), function(x) c(x, sum(is_almost_eq(x, vars))))
-    complements <- vector(mode = "logical", length(vars))
-    for (i in 1:ncol(vars_distinct)) {
-      if (vars_distinct[2, i] < 2)
-        next
-    
-      indices <- which(is_almost_eq(vars_distinct[1, i], vars))
-    
-      for (j in 2:length(indices)) {
-        indices_prev <- indices[1:(j - 1)] 
-        complements[indices_prev] <- 
-          complements[indices_prev] | apply(
-            rulevars[, indices_prev, drop = FALSE] != rulevars[, indices[j]], 2, all)
+  ## Remove complement rules
+  if (removecomplements && rules_to_process) {
+    if (sparse) {
+      is_all_binary <- all(rulevars@x %in% 0:1)
+      if (!is_all_binary) stop("method not implemented for non-binary rules")
+      # find columns which length of non-zero entries are equal to the number 
+      # of rows. The latter are complements if the union of the row indices are
+      # equal to the number of rows
+      js <- rep(1:ncol(rulevars), diff(rulevars@p))
+      is <- rulevars@i + 1
+      n <- rulevars@Dim[1]
+      p <- rulevars@Dim[2]
+      
+      Jfac <- factor(js, levels = 1:p)
+      row_indices <- split(is, Jfac)
+      lengths <- table(Jfac)
+      complements <- logical(p)
+      for(i in 1:(p - 1)) {
+        if (complements[i])
+          next
+        
+        is_potential <- which(lengths[i] + lengths[(i + 1):p] == n) + i
+        is_potential <- is_potential[!complements[is_potential]]
+        
+        if (length(is_potential) == 0)
+          next
+        
+        union_len <- sapply(
+          mapply(union, x = row_indices[i], 
+                 y = row_indices[is_potential], SIMPLIFY = FALSE), 
+          length)
+        is_compl <- which(union_len == n) 
+        if (length(is_compl) > 0) {
+          complements[is_potential[is_compl]] <- TRUE
+        }
       }
-    }  
+        
+    } else { # sparse = FALSE
+      if (!is.logical(rulevars)) {
+        stop("method not implemented for non-binary rules")
+      }
+      
+      # find columns will equal variance to reduce the number of comparisons
+      vars <- apply(rulevars, 2, var_bin)
+      vars_distinct <- lapply(
+        unique(vars), function(x) { 
+          idx <- which(is_almost_eq(x, vars))
+          list(var = x, n = length(idx), idx = idx)
+        })
+      
+      complements <- logical(ncol(rulevars))
+      for (va in vars_distinct) {
+        if(va$n < 2L)
+          next
+        
+        idx <- va$idx
+        idx <- setdiff(idx, which(complements))
+
+        if(length(idx) < 2)
+          next
+        
+        n_idx <- length(idx)
+        for(j in 1:(n_idx - 1)){
+          if (complements[idx[j]])
+            next
+          
+          this_val <- rulevars[, idx[j]]
+          is_compl <- 
+            which(apply(
+              rulevars[, idx[(j + 1):n_idx], drop = FALSE], 2, 
+              function(x) all(x != this_val))) + j
+          
+          if (length(is_compl) > 0)
+            complements[idx[is_compl]] <- TRUE
+        }
+      }
+      
+    }
+    
     complements <- which(complements)
     complements.removed <- rules[complements]
-    if (length(complements) > 0) {
-      rulevars <- rulevars[, -complements, drop = FALSE]
+    if (length(complements) > 0)
       rules <- rules[-complements]
-    }
+    
   } else {
     complements.removed <- NULL
   }
   
+  rulevars = if (keep_rulevars && rules_to_process)
+    rulevars[, names(rules)] else NULL
+  
   ## Return results:
   if (return.dupl.compl) {
-    return(list(rules = rules, 
-                duplicates.removed = duplicates.removed,
-                complements.removed = complements.removed))
-  } else {
-    return(rules)
+    return(list(
+      rules = rules, rulevars = rulevars,
+      duplicates.removed = duplicates.removed,
+      complements.removed = complements.removed))
   }
   
+  list(rules = rules, rulevars = rulevars)
+}
+
+# see https://stackoverflow.com/a/51457395/5861244
+duplicated.dgCMatrix <- function (dgCMat, MARGIN) {
+  MARGIN <- as.integer(MARGIN)
+  n <- nrow(dgCMat)
+  p <- ncol(dgCMat)
+  J <- rep(1:p, diff(dgCMat@p))
+  I <- dgCMat@i + 1
+  x <- dgCMat@x
+  if (MARGIN == 1L) {
+    ## check duplicated rows
+    names(x) <- J
+    RowLst <- split(x, I)
+    is_empty <- setdiff(1:n, I)
+    result <- duplicated.default(RowLst)
+  } else if (MARGIN == 2L) {
+    ## check duplicated columns
+    names(x) <- I
+    ColLst <- split(x, J)
+    is_empty <- setdiff(1:p, J)
+    result <- duplicated.default(ColLst)
+  } else {
+    warning("invalid MARGIN; return NULL")
+    result <- NULL
+  }
+  
+  if(any(is_empty)){
+    out <- logical(if(MARGIN == 1L) n else p)
+    out[-is_empty] <- result
+    if(length(is_empty) > 1)
+      out[is_empty[-1]] <- TRUE
+    result <- out
+  }
+  
+  result
+}
+
+# see https://stackoverflow.com/a/30089750/5861244
+cbind_sparse_vec <- function (...) {
+  args <- list(...)
+  stopifnot(all(sapply(args, inherits, what = "dsparseVector")))
+  
+  un_lengths <- unique(sapply(args,length))
+  stopifnot(length(un_lengths) == 1)
+  
+  return(sparseMatrix( 
+    x = unlist(lapply(args, slot, "x")), 
+    i = unlist(lapply(args, slot ,"i")), 
+    p = c(0, cumsum(sapply(args, function(x) { length(x@x) } ))),
+    dims=c(un_lengths, length(args))))
 }
 
 
@@ -186,8 +299,7 @@ list.rules <- function (x, i = NULL, removecomplements = TRUE, ...) {
       slevels <- slevels[!is.na(slevels)]
       srule <- paste(svar, " %in% c(\"", paste(slevels, 
                                                collapse = "\", \"", sep = ""), "\")", sep = "")
-    }
-    else {
+    } else {
       if (is.null(index)) {
         index <- 1:length(kid)
       }
@@ -310,4 +422,21 @@ get_y_learn_multinomial <- function(eta, y) {
   })
   p <- 1 / (1 + exp(-eta))
   (y - p) / sqrt(p * (1 - p))
+}
+
+
+
+.get_most_sparse_rule <- function(rules, data){
+  #####
+  # we could do this faster by evaluating all the rules at once but we do not
+  # to reduce the memory usage
+  n <- nrow(data)
+  sapply(rules, function(r){
+    x <- eval(parse(text = r), data)
+    if(!is.logical(x))
+      stop("non-rulle is passed")
+    if(sum(x) > n / 2)
+      return(paste0("!(", r, ")"))
+    r
+  })
 }
\ No newline at end of file
