@@ -2,6 +2,7 @@ context("Tests the pre functions")
 
 
 test_that("Get previous results with airquality and pre function", {
+  suppressWarnings(RNGversion("3.1.0"))
   set.seed(42)
   #####
   # With learning rate
@@ -26,6 +27,7 @@ test_that("Get previous results with airquality and pre function", {
   
   #####
   # Works only with rules
+  suppressWarnings(RNGversion("3.1.0"))
   set.seed(42)
   airq.ens <- pre(Ozone ~ ., data = airquality, type="rules", ntrees = 10)
   airq.ens <- airq.ens[!names(airq.ens) %in%  c(
@@ -36,6 +38,7 @@ test_that("Get previous results with airquality and pre function", {
   
   ####
   # Works with multivariate responses
+  suppressWarnings(RNGversion("3.1.0"))
   set.seed(42)
   airq.ens <- pre(Ozone + Solar.R ~ ., data = airquality, family = "mgaussian", ntrees = 10)
   airq.ens <- airq.ens[!names(airq.ens) %in%  c(
@@ -49,6 +52,7 @@ test_that("Get previous results with airquality and pre function", {
 test_that("Get previous results with PimaIndiansDiabetes and pre function", {
   #####
   # Without learning rate
+  suppressWarnings(RNGversion("3.1.0"))
   set.seed(7385056)
   fit <- pre(diabetes ~ ., data = PimaIndiansDiabetes, ntrees = 10, maxdepth = 3)
   
@@ -63,6 +67,7 @@ test_that("Get previous results with PimaIndiansDiabetes and pre function", {
   
   #####
   # With learning rate
+  suppressWarnings(RNGversion("3.1.0"))
   set.seed(4989935)
   fit <- pre(diabetes ~ ., data = PimaIndiansDiabetes, learnrate = .01,
              ntrees = 10, maxdepth = 3)
@@ -81,6 +86,7 @@ test_that("Get previous results with iris and pre function", {
   
   #####
   # With learning rate
+  suppressWarnings(RNGversion("3.1.0"))
   set.seed(7385056)
   fit <- pre(Species ~ ., data = iris, ntrees = 10, maxdepth = 3)
   
@@ -101,6 +107,7 @@ test_that("Get previous results with iris and pre function", {
     ##
     #####
     # Without learning rate
+    suppressWarnings(RNGversion("3.1.0"))
     set.seed(4989935)
     fit <- pre(Species ~ ., data = iris, learnrate = 0, ntrees = 10, maxdepth = 3, nlambda =10)
     
@@ -119,6 +126,7 @@ test_that("Get previous results with iris and pre function", {
     library("doParallel")
     cl <- makeCluster(2)
     registerDoParallel(cl)
+    suppressWarnings(RNGversion("3.1.0"))
     set.seed(4989935)
     fit2 <- pre(Species ~ ., data = iris, learnrate = 0, ntrees = 10, maxdepth = 3, par.init=TRUE, par.final=TRUE, nlambda = 10)
     stopCluster(cl)
@@ -137,6 +145,7 @@ test_that("Get previous results with iris and pre function", {
 
 
 test_that("Get previous results with lung survival data", {
+  suppressWarnings(RNGversion("3.1.0"))
   set.seed(42)
   fit <- pre(Surv(time, status) ~ ., data = Lung, ntrees = 10, family = "cox")
   fit <- fit[names(fit) %in%  c("rules", "glmnet.fit")]
@@ -146,4 +155,43 @@ test_that("Get previous results with lung survival data", {
   fit$rules <- as.matrix(fit$rules)
   # save_to_test(fit, "lung_w_pre_surv")
   expect_equal(fit, read_to_test("lung_w_pre_surv"), tolerance = 1.490116e-06)
-})
\ No newline at end of file
+})
+
+test_that("pre gives almost the same rules with `sparse` set to `TRUE` and `FALSE`", {
+  #####
+  # With learning rate
+  suppressWarnings(RNGversion("3.1.0"))
+  set.seed(seed <- 42)
+  airq.ens <- pre(Ozone ~ ., data=airquality, ntrees = 10)
+  suppressWarnings(RNGversion("3.1.0"))
+  set.seed(seed)
+  airq.ens.sparse <- pre(Ozone ~ ., data=airquality, ntrees = 10, sparse = TRUE)
+  
+  # will differ because sparse complements are used
+  expect_false(isTRUE(all.equal(
+    airq.ens$glmnet.fit$glmnet.fit$beta, 
+    airq.ens.sparse$glmnet.fit$glmnet.fit$beta)))
+  
+  expect_false(all(
+    airq.ens$rules$description == 
+      airq.ens.sparse$rules$description))
+  
+  # but the prediction are identical 
+  expect_equal(predict(airq.ens), predict(airq.ens.sparse))
+  
+  # check the classes of their model matrices
+  expect_equal(class(airq.ens$modmat), "matrix")
+  expect_s4_class(airq.ens.sparse$modmat, "dgCMatrix")
+})
+
+test_that("predict.pre works with `sparse` set to `TRUE`", {
+  #####
+  # With learning rate
+  suppressWarnings(RNGversion("3.1.0"))
+  set.seed(42)
+  airq.ens.sparse <- pre(Ozone ~ ., data=airquality, ntrees = 10, sparse = TRUE)
+  
+  expect_equal(
+    predict(airq.ens.sparse, type = "link", newdata = airquality[1:10, ]),
+    predict(airq.ens.sparse, type = "link")[1:10])
+})
